from cement.core.controller import CementBaseController, expose
from distutils.dir_util import copy_tree
from app.services.dados_service import get_random_words
from os import path

class BaseController(CementBaseController):
    class Meta:
        label = 'base'
        description = 'Random word generator'
        arguments = [
            (['-f', '--files'], {
                'action': 'store',
                'dest': 'files',
                'help': 'word files',
                'required': False
            }),
            (['-c', '--category'], {
                'action': 'store',
                'dest': 'category',
                'help': 'word file',
                'required': False
            }),
            (['-n', '--number'], {
                'action': 'store',
                'dest': 'number',
                'help': 'number of random words',
                'required': False
            })
        ]

    @expose(hide=True)
    def default(self):
        pargs = self.app.pargs
        files = pargs.files
        if not files:
            files = path.join(path.expanduser('~'), '.dados')
            if not path.exists(files):
                copy_tree(path.abspath(path.join(path.dirname(path.realpath(__file__)), '..', 'dados')), files)
        category = pargs.category
        number = pargs.number
        if number:
            number = int(number)
        else:
            number = 1
        random_words = get_random_words(files, category=category, count=number)
        print('\n'.join(random_words))



from cement.core.controller import CementBaseController, expose
from app.services.dados_service import get_categories

class ListController(CementBaseController):
    class Meta:
        label = 'list'
        description = 'list categories'
        stacked_on = 'base'
        stacked_type = 'nested'
        arguments = [
            (['-f', '--files'], {
                'action': 'store',
                'dest': 'files',
                'help': 'word files',
                'required': False
            })
        ]

    @expose(hide=True)
    def default(self):
        pargs = self.app.pargs
        files = pargs.files
        categories = get_categories(files)
        print('\n'.join(categories))



from os import path
from os import listdir
from pydash import _
from future.utils import viewitems
from random import random
from app.exceptions.base_exceptions import DefaultException
import re
import os

def get_words_by_category(filepath):
    words_by_category = {}
    def recurse_words_tree(filepath):
        filepath = path.abspath(path.join(os.getcwd(), filepath))
        if not path.exists(filepath):
            raise DefaultException('\'' + filepath + '\' path does not exist')
        matches = re.findall(r'\/[^\/\.]+', filepath)
        category = matches[len(matches) - 1][1:]
        if not path.isdir(filepath):
            with open(filepath, 'r') as f:
                words = list()
                for line in f.readlines():
                    word = line[:len(line) - 1]
                    if len(word) > 0:
                        words.append(line[:len(line) - 1])
                if category in words_by_category:
                    words_by_category[category] = _.flatten([words, words_by_category[category]])
                else:
                    words_by_category[category] = words
                return { category : f.readlines() }
        words = list()
        for item in listdir(filepath):
            words.append(recurse_words_tree(path.join(filepath, item)))
        return { category: words }
    recurse_words_tree(filepath)
    return words_by_category

def get_words(path, category):
    words_by_category = get_words_by_category(path)
    if category:
        if not category in words_by_category:
            raise DefaultException('\'' + category + '\' category does not exist')
        return words_by_category[category]
    words = list()
    for (key, item) in viewitems(words_by_category):
        for word in words_by_category[key]:
            if not _.includes(words, item):
                words.append(word)
    return words

def get_random_words(path, category=None, count=1):
    words = get_words(path, category)
    if count > len(words):
        count = len(words)
    random_numbers = get_random_numbers(count, len(words))
    random_words = list()
    for random_number in random_numbers:
        random_words.append(words[random_number])
    return random_words

def get_random_numbers(count, length):
    random_numbers = list()
    for i in range(count):
        random_number = int(random() * length)
        if _.includes(random_numbers, random_number):
            random_number = get_next_unique_number(length, random_numbers, random_number)
        random_numbers.append(random_number)
    return random_numbers

def get_next_unique_number(length, numbers, number):
    number = (number + 1) % length
    if _.includes(numbers, number):
        number = get_next_unique_number(length, numbers, number)
    return number

def get_categories(path):
    words_by_category = get_words_by_category(path)
    categories = list()
    for (key, item) in viewitems(words_by_category):
        categories.append(key)
    return categories



from setuptools import setup, find_packages
from codecs import open
from os import path
from app import config

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

install_requires = list()
with open(path.join(here, 'requirements.txt'), encoding='utf-8') as f:
    for line in f.readlines():
        install_requires.append(line)

setup(
    name=config.NAME,

    version=config.VERSION,

    description='Random word generator',

    long_description=long_description,

    url='https://github.com/jamrizzi/foundation',

    author='Jam Risser',

    author_email='jam@jamrizzi.com',

    license='MIT',

    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    keywords='random word dice generator name server generate',

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    install_requires=install_requires,

    include_package_data=True,

    entry_points = {
        'console_scripts': [config.NAME + '=app.__main__:main'],
    }
)



from cement.core.foundation import CementApp
from cement.utils.misc import init_defaults
from config import NAME
from pydash import _
from controllers import (
    BaseController,
    ListController
)
import traceback

defaults = init_defaults(NAME, 'log.logging')
defaults['log.logging']['level'] = 'WARNING'

class App(CementApp):
    class Meta:
        label = NAME
        base_controller = BaseController
        extensions = ['colorlog']
        log_handler = 'colorlog'
        config_defaults=defaults
        handlers = [
            ListController
        ]

def create_app():
    with App() as app:
        return app

def main():
    app = create_app()
    try:
        app.run()
    except BaseException as err:
        if not hasattr(err, 'known') or not err.known:
            return handle_error(app, err)
        if not hasattr(err, 'level'):
            return app.log.error(err.message)
        if err.level == 'warning':
            return app.log.warning(err.message)
        return handle_error(app, err)

def handle_error(app, err):
    debug = False
    if app.config.get('log.logging', 'level') == 'DEBUG':
        debug = True
    if _.includes(app.config.keys(NAME), 'debug') and app.config.get(NAME, 'debug'):
        debug = True
    if debug:
        traceback.print_exc()
    return app.log.error(err.message)

if __name__ == '__main__':
    main()
